function [effic,class,gis,fsel]=classif_TtestFDR_loo(X,y,pat,classif,alpha,loo)
% classif_TtestFDR_loo - a function for classification of 2 groups of 
% subjects; selection of features based on t-test with FDR correction; 
% classification performed using diagonal LDA or linear SVM; correct or
% incorrect leave-one-out cross-validation can be performed
%
% [effic,class,gis,fsel]=classif_TtestFCR_loo(X,y,pat,classif,alpha,loo)
% 
% Input:
% X ...... data matrix (rows correspond to persons, columns to features)
% y ...... row vector with group identifiers
% pat .... identifier of patients
% classif. vector of classifiers which are used (1..diagonal LDA, 2..linear
%          SVM); for example: [1,2] - both classifiers are used; 
%          [2] - linear SVM is used for classification)
% alpha .. threshold for p-values calculated by t-test and corrected using
%          FDR correction
% loo .... binary variable: 0..leave-one-out cross-validation (LOOCV) is 
%          used during data reduction and classification - correct LOOCV; 
%          1..LOOCV is used only during classification - incorrect LOOCV
%
% Output:
% effic .. efficiency of classifiers (i.e. classification performance)
% class .. matrix of votes (group identifiers) which are results of
%          classification algorithms for each person (rows are
%          classifiers, columns are persons)
% gis .... vector with ground truth (true group identifiers)
% fsel ... how many times feature was selected as statistically significant
%
% by Eva Janousova, 29/MAR/2014
%
% ----------------------------------------------------------------------- %

h = waitbar(0,'','Name','t-test with FDR correction');
class=zeros(size(X,1),length(classif))-1; % allocation of a matrix with group identifiers calcuated using classifiers
fsel=zeros(1,size(X,2)); % allocation of a vector with feature selection frequencies
nsel=zeros(1,size(X,1)); % allocation of a vector with number of selected features
gis=y'; % creation of a vector with ground truth (true group identifiers)

% checking the vector with group identifiers:
ytab=tabulate(y);
ytab=ytab(ytab(:,2)>0,1);
if length(ytab)~=2
    error('The number of groups in "y" is not equal to two. Use "help classif_TtestFDR_loo" for more information.')
end
if sum(ytab==pat)==0
    error('The parameter "pat" contains an unvalid identifier. Use "help classif_TtestFDR_loo" for more information.')
end

% data reduction and classification:
switch (loo) 
    case 0 % correct LOOCV
        for i=1:size(X,1)
            waitbar(i/size(X,1),h,sprintf('Classification of data... (%d/%d)',i,size(X,1)));
            ifes=(y==ytab(1)); % selection of subjects from the 1st group
            ifes(i)=0; % testing image is discarded from the 1st group
            inc=(y==ytab(2)); % selection of subjects from the 2nd group
            inc(i)=0; % testing image is discarded from the 2nd group
            [~,pvals]=ttest2(X(ifes,:),X(inc,:)); % two-sample t-test
            [pID,~]=FDR(pvals,alpha); % FDR correction
            if ~isempty(pID) % if there is zero selected features, the subject's group identifier remains equal to -1
                %sum(pvals<=pID)
                fsel=fsel+(pvals<=pID); % adjustment of the vector with feature selection frequencies
                nsel(i)=sum(pvals<=pID); % adjustment of the vector with the number of selected features
                class(i,:)=classifiers(X(i,pvals<=pID),X([1:(i-1),(i+1):end],pvals<=pID),y([1:(i-1),(i+1):end]),classif); % classification based on features with p-value smaller than FDR corrected threshold
            end
        end
        
    case 1 % incorrect LOOCV
        [~,pvals]=ttest2(X(y==ytab(1),:),X(y==ytab(2),:)); % two-sample t-test
        [pID,~]=FDR(pvals,alpha); % FDR correction
        if ~isempty(pID) % if there is zero selected features, group identifiers remains equal to -1
            %sum(pvals<=pID)
            fsel=fsel+(pvals<=pID); % adjustment of the vector with feature selection frequencies
            nsel=sum(pvals<=pID); % adjustment of the vector with the number of selected features
            for i=1:size(X,1)
                waitbar(i/size(X,1),h,sprintf('Classification of data... (%d/%d)',i,size(X,1)));
                class(i,:)=classifiers(X(i,pvals<=pID),X([1:(i-1),(i+1):end],pvals<=pID),y([1:(i-1),(i+1):end]),classif); % classification based on features with p-value smaller than FDR corrected threshold
            end
        end
end

% classification performance:
[effic]=cperformance(class',y,pat);
effic=[ones(length(classif),1)*mean(nsel),ones(length(classif),1)*std(nsel),classif',effic];
%the values of effic are: meanNfeatures,stdNfeatures,classifier,TP,FN,TN,
%FP,accuracy,sensitivity,specificity,precision
close(h);

end
