function [effic,class,gis,fsel]=classif_Ttest_nsel_loo(X,y,pat,classif,nsel,loo)
% classif_Ttest_nsel_loo - a function for classification of 2 groups of 
% subjects; selection of nsel features with lowest p-value based on t-test; 
% classification performed using diagonal LDA or linear SVM; correct or
% incorrect leave-one-out cross-validation can be performed
%
% [effic,class,gis,fsel]=classif_Ttest_nsel_loo(X,y,pat,classif,nsel,loo)
%                          
% Input:
% X ...... data matrix (rows correspond to persons, columns to features)
% y ...... row vector with group identifiers
% pat .... identifier of patients
% classif. vector of classifiers which are used (1..diagonal LDA, 2..linear
%          SVM); for example: [1,2] - both classifiers are used; 
%          [2] - linear SVM is used for classification)
% nsel ... number of most significant features ordered based on t-test
% loo .... binary variable: 0..leave-one-out cross-validation (LOOCV) is 
%          used during data reduction and classification - correct LOOCV; 
%          1..LOOCV is used only during classification - incorrect LOOCV
%
% Output:
% effic .. efficiency of classifiers (i.e. classification performance)
% class .. matrix of votes (group identifiers) which are results of
%          classification algorithms for each person (rows are
%          classifiers, columns are persons)
% gis .... vector with ground truth (true group identifiers)
% fsel ... how many times feature was selected as statistically significant
%
% by Eva Janousova, 04/DEC/2015
%
% ----------------------------------------------------------------------- %

h = waitbar(0,'','Name','t-test');
class=zeros(size(X,1),length(nsel)*length(classif)); % allocation of a matrix with group identifiers calcuated using classifiers
fsel=zeros(length(nsel),size(X,2)); % allocation of a matrix with feature selection frequencies
gis=y'; % creation of a vector with ground truth (true group identifiers)

% checking the vector with group identifiers:
ytab=tabulate(y);
ytab=ytab(ytab(:,2)>0,1);
if length(ytab)~=2
    error('The number of groups in "y" is not equal to two. Use "help classif_Ttest_nsel_loo" for more information.')
end
if sum(ytab==pat)==0
    error('The parameter "pat" contains an unvalid identifier. Use "help classif_Ttest_nsel_loo" for more information.')
end

% data reduction and classification:
switch (loo) 
    case 0 % correct LOOCV
        for i=1:size(X,1)
            waitbar(i/size(X,1),h,sprintf('Classification of data... (%d/%d)',i,size(X,1)));
            ifes=(y==ytab(1)); % selection of subjects from the 1st group
            ifes(i)=0; % testing image is discarded from the 1st group
            inc=(y==ytab(2)); % selection of subjects from the 2nd group
            inc(i)=0; % testing image is discarded from the 2nd group
            [~,pvals]=ttest2(X(ifes,:),X(inc,:)); % two-sample t-test
            [~,inx]=sort(pvals); % sorting the p-values from smallest to largest
            jj = 1; % counter initialization
            for j=1:length(nsel)
                fsel(j,inx(1:nsel(j)))=fsel(j,inx(1:nsel(j)))+1; % adjustment of the matrix with feature selection frequencies
                class(i,(jj:(jj+length(classif)-1)))=classifiers(X(i,inx(1:nsel(j))),X([1:(i-1),(i+1):end],inx(1:nsel(j))),y([1:(i-1),(i+1):end]),classif); % classification based on "nsel" features with smallest p-values
                jj=jj+length(classif); % counter increment
            end
        end
        
    case 1 % incorrect LOOCV
        [~,pvals]=ttest2(X(y==ytab(1),:),X(y==ytab(2),:)); % two-sample t-test
        [~,inx]=sort(pvals); % sorting the p-values from smallest to largest
        jj = 1; % counter initialization
        for j=1:length(nsel)
            fsel(j,inx(1:nsel(j)))=1; % adjustment of the matrix with feature selection frequencies
            classsub=zeros(size(X,1),length(classif)); % allocation of a partial matrix with classifier votes
            for i=1:size(X,1)
                waitbar(i/size(X,1),h,sprintf('Classification of data with %d features... (%d/%d)',nsel(j),i,size(X,1)));
                classsub(i,:)=classifiers(X(i,inx(1:nsel(j))),X([1:(i-1),(i+1):end],inx(1:nsel(j))),y([1:(i-1),(i+1):end]),classif); % classification based on "nsel" features with smallest p-values
            end
            class(:,(jj:(jj+length(classif)-1)))=classsub; % adjustment of the matrix with votes
            jj=jj+length(classif); % counter increment
        end
end

% classification performance:
[effic]=cperformance(class',y,pat);
effic=[ones(length(classif),1)*nsel,zeros(length(classif),1),classif',effic];
%the values of effic are: meanNfeatures,stdNfeatures,classifier,TP,FN,TN,
%FP,accuracy,sensitivity,specificity,precision
close(h);

end
