function [effic,class,gis]=classif_isPCA_loo(X,y,pat,classif,loo,m)
% classif_isPCA_loo - a function for classification of 2 groups of 
% subjects; data reduced using intersubject principal component analysis  
% (isPCA); classification performed using diagonal LDA or linear SVM; 
% correct or incorrect leave-one-out cross-validation can be performed
%
% [effic,class,gis]=classif_isPCA_loo(X,y,pat,classif,loo,m)
% 
% Input:
% X ...... data matrix (rows correspond to persons, columns to features)
% y ...... row vector with group identifiers
% pat .... identifier of patients
% classif. vector of classifiers which are used (1..diagonal LDA, 2..linear
%          SVM); for example: [1,2] - both classifiers are used; 
%          [2] - linear SVM is used for classification)
% loo .... binary variable: 0..leave-one-out cross-validation (LOOCV) is 
%          used during data reduction and classification - correct LOOCV; 
%          1..LOOCV is used only during classification - incorrect LOOCV
% m ...... number of eigenvectors which you want to discard (a possitive 
%          number means discarding m eigenvectors with smallest 
%          eigenvalues, a negative number means discarding m eigenvectors 
%          with largest eigenvalues and zero means discarding no 
%          eigenvectors); absolute value of m must be less than a number of
%          subjects minus one
%
% Output:
% effic .. efficiency of classifiers (i.e. classification performance)
% class .. matrix of votes (group identifiers) which are results of
%          classification algorithms for each person (rows are
%          classifiers, columns are persons)
% gis .... vector with ground truth (true group identifiers)
%
% by Eva Janousova, 23/MAR/2014
%
% ----------------------------------------------------------------------- %

h = waitbar(0,'','Name','PCA');
class=zeros(size(X,1),length(classif)); % allocation of matrix with group identifiers calcuated using classifiers
gis=y'; % creation of a vector with ground truth (true group identifiers)

% checking the vector with group identifiers:
ytab=tabulate(y);
ytab=ytab(ytab(:,2)>0,1);
if length(ytab)~=2
    error('The number of groups in "y" is not equal to two. Use "help classif_isPCA_loo" for more information.')
end
if sum(ytab==pat)==0
    error('The parameter "pat" contains an unvalid identifier. Use "help classif_isPCA_loo" for more information.')
end

% data reduction and classification:
switch (loo) 
    case 0 % correct LOOCV
        for i=1:size(X,1)
            waitbar(i/size(X,1),h,sprintf('Classification of data... (%d/%d)',i,size(X,1)));
            [X_train_red,~,meanX,P]=isPCA_reduc(X([1:(i-1),(i+1):end],:),m); % PCA 
            test=X(i,:);
            test=test-meanX; % subtraction of mean from the test image
            test=test*P; % reduction of the test image
            class(i,:)=classifiers(test,X_train_red,y([1:(i-1),(i+1):end]),classif); % classification
        end
    case 1 % incorrect LOOCV
        [X_red,~,~,~]=isPCA_reduc(X,m); % PCA 
        for i=1:size(X,1)
            waitbar(i/size(X,1),h,sprintf('Classification of data... (%d/%d)',i,size(X,1)));
            class(i,:)=classifiers(X_red(i,:),X_red([1:(i-1),(i+1):end],:),y([1:(i-1),(i+1):end]),classif); % classification
        end
end

% classification performance:
[effic]=cperformance(class',y,pat);
effic=[classif',effic];
%the values of effic are: classifier,TP,FN,TN,FP,accuracy,sensitivity,
%specificity,precision
close(h);

end
