function [effic,class,gis,nit,fsel,nsel]=classif_pLDA_loo(X,y,pat,classif,loo,thresh,lambda,prob,B)
% classif_pLDA_loo - a function for classification of 2 groups of subjects;
% selection of features based on penalised linear discriminant analysis 
% (pLDA) with resampling; classification performed using diagonal LDA or 
% linear SVM; correct or incorrect leave-one-out cross-validation can be 
% performed
%
% [effic,class,gis,nit,fsel,nsel]=classif_pLDA_loo(X,y,pat,classif,loo,
%          thresh,lambda,prob,B)
% 
% Input:
% X ...... data matrix (rows correspond to persons, columns to features)
% y ...... row vector with group identifiers
% pat .... identifier of patients
% classif. vector of classifiers which are used (1..diagonal LDA, 2..linear
%          SVM); for example: [1,2] - both classifiers are used; 
%          [2] - linear SVM is used for classification)
% loo .... binary variable: 0..leave-one-out cross-validation (LOOCV) is 
%          used during data reduction and classification - correct LOOCV; 
%          1..LOOCV is used only during classification - incorrect LOOCV
% thresh . threshold to be used as an assessment of convergence
% lambda . regularization parameter
% prob ... probability threshold(s) for selection of features
% B....... number of iterations in resampling
%
% Output:
% effic .. efficiency of classifiers (i.e. classification performance)
% class .. matrix of votes (group identifiers) which are results of
%          classification algorithms for each person (rows are
%          classifiers, columns are persons)
% gis .... vector with ground truth (true group identifiers)
% nit .... matrix with number of iterations of convergence of penalized LDA
% fsel ... how many times features were selected (in case of incorrect  
%          LOOCV, fsel is equal to percentages calculated using pLDA)
% nsel ... number of selected features in each iteration
%
% by Eva Janousova, 18/AUG/2015
%
% ----------------------------------------------------------------------- %

h = waitbar(0,'Initialization of classification','Name','pLDA with resampling');
class=zeros(size(X,1),length(lambda)*length(prob)*length(classif)); % allocation of a matrix with group identifiers calcuated using classifiers
gis=y'; % creation of a vector with ground truth (true group identifiers)

% checking the vector with group identifiers:
ytab=tabulate(y);
ytab=ytab(ytab(:,2)>0,1);
if length(ytab)~=2
    error('The number of groups in "y" is not equal to two. Use "help classif_pLDA_loo" for more information.')
end
if sum(ytab==pat)==0
    error('The parameter "pat" contains an unvalid identifier. Use "help classif_pLDA_loo" for more information.')
end

% data reduction and classification:
switch (loo) 
    case 0 % correct LOOCV
        fsel=zeros(length(lambda)*length(prob),size(X,2)); % allocation of matrix with feature selection frequencies
        nsel=zeros(length(lambda)*length(prob),size(X,1)); % allocation of a matrix with a number of selected features in each iteration
        for i=1:size(X,1)
            waitbar(i/size(X,1),h,sprintf('Classification of subject %d for lambda=%0.2f ...',i,lambda(1)));
            [percsel,nit]=stability_pLDA(X([1:(i-1),(i+1):end],:),y([1:(i-1),(i+1):end]),B,lambda,thresh); % pLDA with resampling
            jj = 1; % counter initialization
            for e=1:length(lambda)
                waitbar(i/size(X,1),h,sprintf('Classification of subject %d for lambda=%0.2f ...',i,lambda(e)));
                for j=1:length(prob)
                    sel = (sum(percsel(e,:)>=prob(j),1)>0); % selected features with probability of selection higher or equal prob(j)
                    fsel((e-1)*length(prob)+j,:)=fsel((e-1)*length(prob)+j,:)+sel; % adjustment of the matrix with feature selection frequencies
                    nsel((e-1)*length(prob)+j,i)=sum(sel); % adjustment of the matrix with the number of selected features in each iteration
                    class(i,(jj:(jj+length(classif)-1)))=classifiers(X(i,sel),X([1:(i-1),(i+1):end],sel),y([1:(i-1),(i+1):end]),classif); % classification based on the selected features
                    jj=jj+length(classif); % counter increment
                end
            end
        end
        res=[kron(lambda',ones(length(prob),1)), repmat(prob',length(lambda),1), mean(nsel,2), std(nsel,0,2), kron(mean(nit,2),ones(length(prob),1)), kron(std(nit,0,2),ones(length(prob),1))]; % creation of a matrix with basic summary
   
    case 1 % incorrect LOOCV
        [perc,nit]=stability_pLDA(X,y,B,lambda,thresh); % pLDA with resampling
        res = zeros(length(lambda)*length(prob),6); % allocation of a matrix with basic summary
        nsel=zeros(length(lambda)*length(prob),1); % allocation of a vector with a number of selected features in each iteration
        fsel=perc; % creation of a matrix with feature selection frequencies
        jj = 1; % counter initialization
        for e=1:length(lambda)
            waitbar(0/size(X,1),h,sprintf('Classification of subject %d for lambda=%0.2f ...',1,lambda(e)));
            for j=1:length(prob)
                sel = (sum(perc(e,:)>=prob(j),1)>0); % selected features with probability of selection higher or equal prob(j)
                nsel((e-1)*length(prob)+j)=sum(sel); % adjustment of the matrix with the number of selected features in each iteration
                res((e-1)*length(prob)+j,:)=[lambda(e),prob(j),sum(sel),0,mean(nit(e,:)),std(nit(e,:))]; % adjustment of the matrix with basic summary
                classsub=zeros(size(X,1),length(classif)); % allocation of a partial matrix with classifier votes
                for i=1:size(X,1)
                    waitbar(i/size(X,1),h,sprintf('Classification of subject %d for lambda=%0.2f ...',i,lambda(e)));
                    classsub(i,:)=classifiers(X(i,sel),X([1:(i-1),(i+1):end],sel),y([1:(i-1),(i+1):end]),classif); % classification based on the selected features
                end
                class(:,(jj:(jj+length(classif)-1)))=classsub; % adjustment of the matrix with votes
                jj=jj+length(classif); % counter increment
            end
        end   
end

% classification performance:
[effic]=[kron(res,ones(length(classif),1)),repmat((1:length(classif))',length(lambda)*length(prob),1),cperformance(class',y,pat)]; 
%the values of effic are: lambda,prob,meanNfeatures,stdNfeatures,meanNiter,
%stdNiter,classifier,TP,FN,TN,FP,accuracy,sensitivity,specificity,precision
close(h);

end
