function [V,pvar]=isPCA_eig(Xc,m)
% isPCA_eig - a function for computation of the projection matrix of   
% intersubject principal component analysis (isPCA); isPCA is based on   
% computation of eigenvectors of covariance matrix of features from   
% eigenvectors of covariance matrix of subjects. The function is a  
% modification of function pca.m created by Carlos Thomaz, DoC-IC/London, 
% 19/jan/2004.
%
% [V,pvar]=isPCA_eig(Xc,m)
%
% Input:
% Xc ..... centered data matrix (where rows correspond to persons and 
%          colums to features)
% m ...... number of eigenvectors which you want to discard (a possitive 
%          number means discarding m eigenvectors with smallest 
%          eigenvalues, a negative number means discarding m eigenvectors 
%          with largest eigenvalues and zero means discarding no 
%          eigenvectors); absolute value of m must be less than a number of
%          images minus one
%
% Output:
% V ...... isPCA projection matrix which consists of eigenvectors
% pvar ... explained variability by isPCA
%
% by Eva Janousova, 27/AUG/2010.


%------------------------------------------------------------------------%

% verify number of eigenvectors which you want to discard is correct:
n = size(Xc,1); % number of subjects
if (abs(m)>=(n-1))
    error('Number of eigenvectors which you want to discard is incorrect! Use "help recog" for more information.');
end

% compute covariance matrix of subjects and its eigenvectors and eigenvalues:
C = Xc* Xc'./(n-1); % covariance matrix of subjects
[Qv,Qa] = eig(C, 'nobalance'); % computation of eigenvectors Qv and eigenvalues Qa of the covariance matrix of subjects

% sort eigenvalues from largest to smallest and sort their corresponding eigenvectors:
[Ks,Ki] = sort(diag(Qa));
Ki = flipud(Ki); 
Qv = Qv(:,Ki(1:(n-1))); % sorted n-1 eigenvectors
Qa= flipud(Ks(2:end)); % sorted n-1 eigenvalues

% discard eigenvectors:
if m>=0
    Qv_s=Qv(:,1:(n-1-m)); % discarding m eigenvectors with smallest eigenvalues
    Qa_s=Qa(1:(n-1-m)); % discarding m smallext eigenvalues
elseif m<0
    m=-m;
    Qv_s=Qv(:,(m+1):end); % discarding m eigenvectors with largest eigenvalues
    Qa_s=Qa((m+1):end); % discarding m largest eigenvalues
end
pvar=100*sum(Qa_s)/sum(Qa); % explained variability

% trasform eigenvectors of covariance matrix of subjects into eigenvectors of covariance matrix of features:
V = zeros(size(Xc,2),n-1-abs(m)); % allocation of a projection matrix V with eigenvectors of covariance matrix of features
for i = 1:(n-1-abs(m))
    V(:,i)=(Xc' * Qv_s(:,i))./(sqrt(Qa_s(i)*(n-1))); % transformation
end

end