function [perc,nit]=stability_pLDA(X,y,B,lambda,thresh)
% stability_pLDA - a function for feature selection using penalised linear 
% discriminant analysis (pLDA) with resampling
% 
% [perc,nit]=stability_pLDA(X,y,B,lambda,thresh)
%
% Input: 
% X ...... data matrix (rows correspond to persons, columns to features)
% y ...... row vector with group identifiers
% B ...... number of iterations in resampling
% lambda . regularization parameter
% thresh . threshold to be used as an assessment of convergence
%
% Output:
% perc ... percentages of how often feature was selected to play role in
%          pLDA model
% nit .... matrix with number of iterations of convergence of penalized LDA
%
% NOTE: 
% The function is now designed for analysis of 2 groups of subjects only!
%
% by Eva Janousova, 22/MAY/2011.

%------------------------------------------------------------------------%

% initial settings:
[~,p] = size(X); % number of predictors (features)
if (size(y,2)>size(y,1))
    y=y'; % y must be a column vector
end
groups = tabulate(y); % a frequency table of group identifiers for detection of a number of groups of images
groups = groups((groups(:,2)>0),1); % creating a vector with values of identifiers
inx1 = find(y==groups(1)); % indexes of subjects from the 1st group
inx2 = find(y==groups(2)); % indexes of subjects from the 2nd group
perc = zeros(length(lambda),p); % allocation of a matrix with percentages of how many times the feature was selected
nit = zeros(length(lambda),B); % allocation of a matrix with number of iterations of convergence of pLDA

% checking values of the parameter lambda:
if (min(lambda) < 0)
    error('Lambda should be non-negative!')
end

% recoding y into a vector with 0 and 1:
y_code = zeros(size(y));
y_code(y==groups(2))=1;

% stability selection + pLDA:
for b=1:B % number of iterations of stability selection
    
    %selection of training data (i.e. random selection of half of data)
    inxr1 = randperm(length(inx1)); % random permutation of numbers from 1 to length(inx1)
    inxr2 = randperm(length(inx2)); % random permutation of numbers from 1 to length(inx2)
    inx_half = [inx1(inxr1(1:floor(length(inx1)/2)));inx2(inxr2(1:floor(length(inx2)/2)))]; % random selection of half of subjects from the 1st group and half of subjects from the 2nd group
    X_train = X(inx_half,:); % selection of training data
    y_train = y_code(inx_half); % selection of identifiers of training data
    
    %sample size in each group
    n1=sum(y_train==1);
    n2=sum(y_train==0);
    
    %mean vectors for each group (of dimension qx1)
    m1=mean(X_train(y_train==1,:));
    m1=m1';
    m2=mean(X_train(y_train==0,:));
    m2=m2';
    
    %the within variance vector s=(s1,...,sq)
    s=sqrt(sum((X_train(y_train==1,:)-ones(n1,1)*m1').^2)+sum((X_train(y_train==0,:)-ones(n2,1)*m2').^2));
    s=s';
    
    perc_l=zeros(length(lambda),p); % allocation of a matrix with percentages of how often feature was selected to play role in pLDA model
    
    for l=1:length(lambda)
        [v,niters]=pLDA(m1,m2,s,lambda(l),thresh); % pLDA
        perc_l(l,:)=(v~=0); % an indicator variable (1..feature has non-zero coefficient in pLDA model, 0..otherwise)
        nit(l,b)=niters; % adjustment of a matrix with number of iterations of convergence of pLDA
    end
    perc=perc+perc_l; % a sum of indicator variables
end

perc=perc/B; % division of the sum of indicator variables by the number of iterations to obtain percentages

end


